Attribute VB_Name = "m_CodeTiming"
'//**************************************************************************
'// ----------------- Module -----------------
'// Name        : Code timing utility
'// Version     :
'// Author      : Benoit Frigon
'// Created on  : 09-MAY-2002
'// Last update : 09-MAY-2002
'// File        : m_CodeTiming.bas
'// Desc.       :
'//**************************************************************************
'// All rights reserved@Logiciels M.T.L enr. NEQ# 22-48153829(Qubec)
'//**************************************************************************
Option Explicit



'//**************************************************************************
'// Api declares
'//**************************************************************************
Public Declare Function QueryPerformanceCounter Lib "kernel32" (lpPerformanceCount As LARGE_INTEGER) As Long
Public Declare Function QueryPerformanceFrequency Lib "kernel32" (lpFrequency As LARGE_INTEGER) As Long

Public Type LARGE_INTEGER
    lowpart As Long
    highpart As Long
End Type


'//**************************************************************************
'// Variables
'//**************************************************************************
Private m_Initialized As Boolean
Private m_Frequency As Double       '// Frequency of the hi-res counter
Private m_Start As LARGE_INTEGER    '// Hi-res timer value (Before timing)
Private m_End As LARGE_INTEGER      '// Hi-res timer value (After timing)
Private m_CallingDelay As Double    '// Time required to call the stoptiming function
Private m_isTiming As Boolean       '// Timing in progress flag
Private m_TimingComplete As Boolean


'//**************************************************************************
'// Const
'//**************************************************************************
'//--- Conversion units ---
Public Const Units_MilliSeconds = &H0
Public Const Units_MicroSeconds = &H1
Public Const Units_Seconds = &H2
Public Const Units_NoConversion = &H3
Public Enum UnitsType
    Milliseconds = Units_MilliSeconds
    MicroSeconds = Units_MicroSeconds
    Seconds = Units_Seconds
    NoConversion = Units_NoConversion
End Enum



'//**************************************************************************
'// Timing functions
'//**************************************************************************
Public Function InitializeTimer() As Boolean
    
    '//**** Get the high resolution counter frequency (Beats per seconds) ****
    Dim iFreq As LARGE_INTEGER
    If (QueryPerformanceFrequency(iFreq) = 0) Then
        
        '//**** There performance counter doesn't work... ****
        Exit Function
    End If
    m_Frequency = LargeIntToDouble(iFreq)
    m_Initialized = True
    
    InitializeTimer = True
End Function
Public Sub StartTiming()
    If (m_isTiming) Then Exit Sub
    
    '//**** If the timer is not initialized, initialize it ****
    If (Not m_Initialized) Then
        If (Not InitializeTimer) Then Exit Sub
    End If
    
    '//**** Calculate the time required to call the StopTiming function ****
    m_isTiming = True
    Call QueryPerformanceCounter(m_Start)
    m_CodeTiming.StopTiming
    m_CallingDelay = (LargeIntToDouble(m_End) - LargeIntToDouble(m_Start))
        
    '//**** Read the current value of the hi-res timer ****
    m_isTiming = True
    Call QueryPerformanceCounter(m_Start)   '//**** This MUST be the last line in the sub ****
End Sub
Public Sub StopTiming()
    If (Not m_isTiming) Then Exit Sub
    
    '//**** Read the current value of the hi-res timer ****
    Call QueryPerformanceCounter(m_End)
    
    m_isTiming = False
    m_TimingComplete = True
End Sub
Public Function ReadCodeTiming(Optional Units As UnitsType = Units_MicroSeconds) As Double
    Dim nCounts As Long
    nCounts = (LargeIntToDouble(m_End) - LargeIntToDouble(m_Start)) - m_CallingDelay
    If (nCounts < 0) Then nCounts = 0
    
    Select Case Units
        Case Units_MicroSeconds
            ReadCodeTiming = (nCounts / (m_Frequency / 1000000))
        
        Case Units_MilliSeconds
            ReadCodeTiming = (nCounts / (m_Frequency / 1000))
        
        Case Units_Seconds
            ReadCodeTiming = (nCounts / m_Frequency)
        
        Case Else
            ReadCodeTiming = nCounts
    End Select
End Function



'//**************************************************************************
'// Conversion functions
'//**************************************************************************
Public Function LargeIntToDouble(LINT As LARGE_INTEGER) As Double
    Const Shift32bit = (4# * 1024# * 1024# * 1024#)
        
    Dim LoInt As Double
    LoInt = LINT.lowpart
    If (LoInt < 0) Then
        LoInt = (LoInt + Shift32bit)
    End If
    
    LargeIntToDouble = (LINT.highpart * Shift32bit) + LoInt
End Function
